@@ -1,15 +1,14 @@
 '''
-Created on 28/03/2014
+Base classes for Dirty Models
+'''
 
-@author: alfred
-'''
 import weakref
 
 
-class BaseData():
+class BaseData:
 
     """
-    Base class for data inside dirty model
+    Base class for data inside dirty model.
     """
 
     __slots__ = []
