@@ -1,23 +1,28 @@
 """
-fields.py
+Fields to be used with dirty models.
+"""
 
-Fields to be used with dirty_models
-"""
-from datetime import datetime, date, time
+from datetime import datetime, date, time, timedelta
 from dateutil.parser import parse as dateutil_parse
 from .model_types import ListModel
 from collections import Mapping
 
 
+__all__ = ['IntegerField', 'FloatField', 'BooleanField', 'StringField', 'StringIdField',
+           'TimeField', 'DateField', 'DateTimeField', 'TimedeltaField', 'ModelField', 'ArrayField',
+           'HashMapField', 'BlobField', 'MultiTypeField']
+
+
 class BaseField:
 
     """Base field descriptor."""
 
-    def __init__(self, name=None, alias=None, getter=None, setter=None, read_only=False, doc=None):
+    def __init__(self, name=None, alias=None, getter=None, setter=None, read_only=False, default=None, doc=None):
         self._name = None
         self.name = name
         self.alias = alias
         self.read_only = read_only
+        self.default = default
         self._getter = getter
         self._setter = setter
         self.__doc__ = doc or self.get_field_docstring()
@@ -125,9 +130,9 @@ class FloatField(BaseField):
         return isinstance(value, float)
 
     def can_use_value(self, value):
-        return isinstance(value, int) \
-            or (isinstance(value, str)
-                and value.replace('.', '', 1).isnumeric())
+        return isinstance(value, int) or \
+            (isinstance(value, str) and
+                value.replace('.', '', 1).isnumeric())
 
 
 class BooleanField(BaseField):
@@ -166,7 +171,7 @@ class StringField(BaseField):
 
 class StringIdField(StringField):
 
-    """It allows to use a stringId as value in a field."""
+    """It allows to use a string as value in a field, but not allows empty strings"""
 
     def set_value(self, obj, value):
         """Sets value to model if not empty"""
@@ -183,6 +188,16 @@ class DateTimeBaseField(BaseField):
     date_parsers = {}
 
     def __init__(self, parse_format=None, **kwargs):
+        """
+
+        :param parse_format: String format to cast string to datetime. It could be
+            an string format or a :class:`dict` with two keys:
+
+                * ``parser`` key to set how string must be parsed. It could be a callable.
+                * ``formatter`` key to set how datetime must be formatted. It could be a callable.
+
+        :type parse_format: str or dict
+        """
         super(DateTimeBaseField, self).__init__(**kwargs)
         self._parse_format = None
         self.parse_format = parse_format
@@ -194,7 +209,7 @@ class DateTimeBaseField(BaseField):
 
     @property
     def parse_format(self):
-        """Model_class getter: model class used on field"""
+        """Parse_format getter: datetime format used on field"""
         return self._parse_format
 
     @parse_format.setter
@@ -203,24 +218,73 @@ class DateTimeBaseField(BaseField):
         self._parse_format = value
 
     def get_parsed_value(self, value):
-        try:
-            parser = self.date_parsers.get(self.parse_format, {}).get('parser')
-            if callable(parser):
-                return parser(value)
-            return datetime.strptime(value, parser)
-        except:
-            return datetime.strptime(value, self.parse_format)
+        """
+        Helper to cast string to datetime using :member:`parse_format`.
+
+        :param value: String representing a datetime
+        :type value: str
+        :return: datetime
+        """
+
+        def get_parser(parser_desc):
+            try:
+                return parser_desc['parser']
+            except TypeError:
+                try:
+                    return get_parser(self.date_parsers[parser_desc])
+                except KeyError:
+                    return parser_desc
+            except KeyError:
+                pass
+
+        parser = get_parser(self.parse_format)
+
+        if parser is None:
+            try:
+                return dateutil_parse(value)
+            except ValueError:
+                return None
+
+        if callable(parser):
+            return parser(value)
+        return datetime.strptime(value, parser)
 
     def get_formatted_value(self, value):
-        try:
-            formatter = self.date_parsers.get(self.parse_format, {}).get('formatter')
-            if callable(formatter):
-                return formatter(value)
+        """
+        Returns a string from datetime using :member:`parse_format`.
 
-            return value.strftime(format=formatter)
-        except:
+        :param value: Datetime to cast to string
+        :type value: datetime
+        :return: str
+        """
+        def get_formatter(parser_desc):
+            try:
+                return parser_desc['formatter']
+            except TypeError:
+                if isinstance(parser_desc, str):
+                    try:
+                        return get_formatter(self.date_parsers[parser_desc])
+                    except KeyError:
+                        return parser_desc
+                else:
+                    pass
+            except KeyError:
+                try:
+                    if isinstance(parser_desc['parser'], str):
+                        return parser_desc['parser']
+                except KeyError:
+                    pass
+
+        formatter = get_formatter(self.parse_format)
+
+        if formatter is None:
             return str(value)
 
+        if callable(formatter):
+            return formatter(value)
+
+        return value.strftime(format=formatter)
+
 
 class TimeField(DateTimeBaseField):
 
@@ -240,7 +304,7 @@ class TimeField(DateTimeBaseField):
                     return value.time()
 
                 return self.convert_value(self.get_parsed_value(value))
-            except ValueError:
+            except:
                 return None
         elif isinstance(value, datetime):
             return value.timetz()
@@ -270,7 +334,7 @@ class DateField(DateTimeBaseField):
                     return value.date()
 
                 return self.convert_value(self.get_parsed_value(value))
-            except(TypeError, ValueError):
+            except:
                 return None
         elif isinstance(value, datetime):
             return value.date()
@@ -299,7 +363,7 @@ class DateTimeField(DateTimeBaseField):
                     return dateutil_parse(value)
 
                 return self.get_parsed_value(value)
-            except(TypeError, ValueError):
+            except:
                 return None
         elif isinstance(value, date):
             return datetime(year=value.year, month=value.month,
@@ -312,6 +376,20 @@ class DateTimeField(DateTimeBaseField):
         return isinstance(value, (int, str, date, dict, list))
 
 
+class TimedeltaField(BaseField):
+    """It allows to use a timedelta as value in a field."""
+
+    def convert_value(self, value):
+        if isinstance(value, (int, float)):
+            return timedelta(seconds=value)
+
+    def check_value(self, value):
+        return type(value) is timedelta
+
+    def can_use_value(self, value):
+        return isinstance(value, (int, float))
+
+
 class ModelField(BaseField):
 
     """
@@ -405,11 +483,15 @@ class InnerFieldTypeMixin:
 class ArrayField(InnerFieldTypeMixin, BaseField):
 
     """
-    It allows to create a ListModel (iterable in dirty_models.types) of different elements according
+    It allows to create a ListModel (iterable in :mod:`dirty_models.types`) of different elements according
     to the specified field_type. So it is possible to have a list of Integers, Strings, Models, etc.
     When using a model with no specified model_class the model inside field.
     """
 
+    def __init__(self, autolist=False, **kwargs):
+        self._autolist = autolist
+        super(ArrayField, self).__init__(**kwargs)
+
     def get_field_docstring(self):
         if self.field_type:
             return 'Array of {0}'.format(self.field_type.get_field_docstring())
@@ -422,7 +504,11 @@ class ArrayField(InnerFieldTypeMixin, BaseField):
             if not self.field_type.check_value(element) and self._field_type.can_use_value(element):
                 return self.field_type.convert_value(element)
             return element
-        return ListModel([convert_element(element) for element in value], field_type=self.field_type)
+
+        if isinstance(value, (set, list, tuple, ListModel)):
+            return ListModel([convert_element(element) for element in value], field_type=self.field_type)
+        elif self.autolist:
+            return ListModel([convert_element(value)], field_type=self.field_type)
 
     def check_value(self, value):
         if not isinstance(value, ListModel) or not isinstance(value.get_field_type(), type(self.field_type)):
@@ -430,19 +516,41 @@ class ArrayField(InnerFieldTypeMixin, BaseField):
         return True
 
     def can_use_value(self, value):
-        if isinstance(value, (set, list, ListModel)):
+        if isinstance(value, (set, list, tuple, ListModel)):
             if len(value) == 0:
                 return True
             for item in value:
                 if self.field_type.can_use_value(item) or self.field_type.check_value(item):
                     return True
             return False
+        elif self.autolist and (self.field_type.check_value(value) or self.field_type.can_use_value(value)):
+            return True
         else:
             return False
 
+    @property
+    def autolist(self):
+        """
+        autolist getter: autolist flag allows to convert a simple item on a list with
+        one item.
+        """
+        return self._autolist
+
+    @autolist.setter
+    def autolist(self, value):
+        """
+        autolist setter: autolist flag allows to convert a simple item on a list with
+        one item.
+        """
+        self._autolist = value
+
 
 class HashMapField(InnerFieldTypeMixin, ModelField):
 
+    """
+    It allows to create a field which contains a hash map.
+    """
+
     def __init__(self, model_class=None, **kwargs):
         if model_class is None:
             from dirty_models.models import HashMapModel
@@ -455,4 +563,69 @@ class HashMapField(InnerFieldTypeMixin, ModelField):
 
 
 class BlobField(BaseField):
+
+    """
+    It allows any type of data.
+    """
     pass
+
+
+class MultiTypeField(BaseField):
+
+    """
+    It allows to define multiple type for a field. So, it is possible to define a field as
+    a integer and as a model field, for example.
+    """
+
+    def __init__(self, field_types=None, **kwargs):
+        self._field_types = []
+
+        field_types = field_types or []
+
+        for field_type in field_types:
+            if isinstance(field_type, tuple):
+                field_type = field_type[0](**field_type[1])
+            self._field_types.append(field_type if field_type else BaseField())
+        super(MultiTypeField, self).__init__(**kwargs)
+
+    def get_field_docstring(self):
+        if len(self._field_types):
+            return 'Multiple type values allowed:\n{0}'.format("\n".join(["* {0}".format(field.get_field_docstring())
+                                                                          for field in self._field_types]))
+
+    def export_definition(self):
+        result = super(MultiTypeField, self).export_definition()
+        result['field_types'] = [(field_type.__class__, field_type.export_definition())
+                                 for field_type in self._field_types]
+        return result
+
+    def convert_value(self, value):
+        for ft in self._field_types:
+            if ft.can_use_value(value):
+                return ft.convert_value(value)
+
+    def check_value(self, value):
+        for ft in self._field_types:
+            if ft.check_value(value):
+                return True
+        return False
+
+    def can_use_value(self, value):
+        for ft in self._field_types:
+            if ft.can_use_value(value):
+                return True
+        return False
+
+    def get_field_type_by_value(self, value):
+        for ft in self._field_types:
+            if ft.check_value(value):
+                return ft
+        for ft in self._field_types:
+            if ft.can_use_value(value):
+                return ft
+
+        raise TypeError("Value `{0}` can not be used on field `{1}`".format(value, self.name))
+
+    @property
+    def field_types(self):
+        return self._field_types.copy()
