@@ -1,57 +1,101 @@
 """
-models.py
-
-Base model for dirty_models.
+Base models for dirty_models.
 """
 
-from .fields import BaseField, ModelField, ArrayField
-from dirty_models.fields import IntegerField, FloatField, BooleanField, StringField, DateTimeField
-from datetime import datetime
-from dirty_models.model_types import ListModel
-from dirty_models.base import BaseData, InnerFieldTypeMixin
 import itertools
+from datetime import datetime
+
 from collections import Mapping
+from copy import deepcopy
+
+from .base import BaseData, InnerFieldTypeMixin
+from .fields import IntegerField, FloatField, BooleanField, StringField, DateTimeField, \
+    BaseField, ModelField, ArrayField
+from .model_types import ListModel
+
+
+__all__ = ['BaseModel', 'DynamicModel', 'FastDynamicModel', 'HashMapModel']
 
 
 class DirtyModelMeta(type):
-
     """
     Metaclass for dirty_models. It sets automatic fieldnames and
     automatic model_class for ModelField fields.
     """
 
-    def __init__(self, name, bases, classdict):
-        super(DirtyModelMeta, self).__init__(name, bases, classdict)
+    def __init__(cls, name, bases, classdict):
+        super(DirtyModelMeta, cls).__init__(name, bases, classdict)
 
-        fields = {key: field for key, field in self.__dict__.items()}
+        fields = {key: field for key, field in cls.__dict__.items()}
+        structure = {}
         read_only_fields = []
         for key, field in fields.items():
             if isinstance(field, BaseField):
-                self.process_base_field(field, key)
+                cls.process_base_field(field, key)
+                structure[field.name] = field
                 if field.read_only:
                     read_only_fields.append(field.name)
 
-        self._read_only_fields = read_only_fields
+        cls._structure = structure
+        default_data = {}
+        for p in bases:
+            try:
+                default_data.update(deepcopy(p._default_data))
+            except AttributeError:
+                pass
 
-    def process_base_field(self, field, key):
+        default_data.update(deepcopy(cls._default_data))
+        default_data.update({f.name: f.default for f in structure.values() if f.default is not None})
+        cls._default_data = default_data
+
+    def process_base_field(cls, field, key):
         """
-        Preprocess class fields.
+        Preprocess field instances.
+
+        :param field: Field object
+        :param key: Key where field was found
         """
         if not field.name:
             field.name = key
         elif key != field.name:
-            setattr(self, field.name, field)
-        if isinstance(field, ModelField) and not field.model_class:
-            field.model_class = self
-            field.__doc__ = field.get_field_docstring()
-        if isinstance(field, ArrayField) and isinstance(field.field_type, ModelField) \
-                and not field.field_type.model_class:
-            field.field_type.model_class = self
-            field.field_type.__doc__ = field.field_type.get_field_docstring()
+            setattr(cls, field.name, field)
+
+        cls.prepare_field(field)
 
         if field.alias:
             for alias_name in field.alias:
-                setattr(self, alias_name, field)
+                setattr(cls, alias_name, field)
+
+    def prepare_field(cls, field):
+        if isinstance(field, ModelField) and not field.model_class:
+            field.model_class = cls
+            field.__doc__ = field.get_field_docstring()
+
+        try:
+            cls.prepare_field(field.field_type)
+            field.field_type.__doc__ = field.field_type.get_field_docstring()
+        except AttributeError:
+            pass
+
+        try:
+            for inner_field in field.field_types:
+                cls.prepare_field(inner_field)
+        except AttributeError:
+            pass
+
+
+class CamelCaseMeta(DirtyModelMeta):
+
+    """
+    Metaclass for dirty_models. Sets camel case version of field's name as default field name.
+    """
+
+    def process_base_field(self, field, key):
+        from .utils import underscore_to_camel
+
+        if not field.name:
+            field.name = underscore_to_camel(key)
+        super(CamelCaseMeta, self).process_base_field(field, key)
 
 
 def recover_model_from_data(model_class, original_data, modified_data, deleted_data):
@@ -69,7 +113,6 @@ def recover_model_from_data(model_class, original_data, modified_data, deleted_d
 
 
 class BaseModel(BaseData, metaclass=DirtyModelMeta):
-
     """
     Base model with dirty feature. It stores original data and saves
     modifications in other side.
@@ -78,6 +121,8 @@ class BaseModel(BaseData, metaclass=DirtyModelMeta):
     _modified_data = None
     _deleted_fields = None
 
+    _default_data = {}
+
     def __init__(self, data=None, flat=False, *args, **kwargs):
         super(BaseModel, self).__init__(*args, **kwargs)
         self._original_data = {}
@@ -85,6 +130,7 @@ class BaseModel(BaseData, metaclass=DirtyModelMeta):
         self._deleted_fields = []
 
         self.unlock()
+        self.import_data(self._default_data)
         if isinstance(data, (dict, Mapping)):
             self.import_data(data)
         self.import_data(kwargs)
@@ -122,7 +168,7 @@ class BaseModel(BaseData, metaclass=DirtyModelMeta):
             else:
                 self._modified_data[name] = value
                 self._prepare_child(value)
-                if name in self._read_only_fields:
+                if name in self._structure and self._structure[name].read_only:
                     try:
                         value.set_read_only(True)
                     except AttributeError:
@@ -195,7 +241,7 @@ class BaseModel(BaseData, metaclass=DirtyModelMeta):
                 data = data.export_data()
             if isinstance(data, (dict, Mapping)):
                 for key, value in data.items():
-                    if hasattr(self, key):
+                    if not key.startswith('__') and hasattr(self, key):
                         setattr(self, key, value)
 
     def import_deleted_fields(self, data):
@@ -207,13 +253,14 @@ class BaseModel(BaseData, metaclass=DirtyModelMeta):
                 data = [data]
             if isinstance(data, list):
                 for key in data:
-                    if hasattr(self, key):
-                        delattr(self, key)
-                    else:
-                        keys = key.split('.', 1)
-                        if len(keys) == 2:
-                            child = getattr(self, keys[0])
-                            child.import_deleted_fields(keys[1])
+                    if not key.startswith('__'):
+                        if hasattr(self, key):
+                            delattr(self, key)
+                        else:
+                            keys = key.split('.', 1)
+                            if len(keys) == 2:
+                                child = getattr(self, keys[0])
+                                child.import_deleted_fields(keys[1])
 
     def export_data(self):
         """
@@ -300,6 +347,7 @@ class BaseModel(BaseData, metaclass=DirtyModelMeta):
         """
         Pass all the data from modified_data to original_data
         """
+
         def flat_field(value):
             """
             Flat field data
@@ -387,7 +435,7 @@ class BaseModel(BaseData, metaclass=DirtyModelMeta):
         return iterfunc()
 
     def _can_write_field(self, name):
-        return (name not in self._read_only_fields and not self.get_read_only()) or \
+        return name not in self._structure or (not self._structure[name].read_only and not self.get_read_only()) or \
             not self.is_locked()
 
     def _update_read_only(self):
@@ -398,7 +446,12 @@ class BaseModel(BaseData, metaclass=DirtyModelMeta):
                 pass
 
     def __str__(self):
-        return self.__class__.__name__ + '({0})'.format(str(self.export_data()))
+        return '{0}({1})'.format(self.__class__.__name__,
+                                 ",".join(["'{0}': {1}".format(field, repr(self.get_field_value(field)))
+                                           for field in sorted(self.get_fields())]))
+
+    def __repr__(self):
+        return str(self)
 
     @classmethod
     def get_field_obj(cls, name):
@@ -453,9 +506,16 @@ class BaseModel(BaseData, metaclass=DirtyModelMeta):
             else:
                 self.reset_field_value(field)
 
+    @classmethod
+    def get_structure(cls):
+        """
+        Returns a dictionary with model field objects.
+        :return: dict
+        """
+        return cls._structure.copy()
+
 
 class BaseDynamicModel(BaseModel):
-
     """
 
     """
@@ -467,19 +527,6 @@ class BaseDynamicModel(BaseModel):
         except AttributeError:
             return self.get_field_value(name)
 
-    def __reduce__(self):
-        """
-        Reduce function to allow dumpable by pickle
-        """
-        return recover_model_from_data, (self.__class__, self.export_original_data(),
-                                         self.export_modified_data(), self.export_deleted_fields(),)
-
-    def copy(self):
-        """
-        Creates a copy of model
-        """
-        return self.__class__(data=self.export_data())
-
     def _get_field_type(self, key, value):
         """
         Helper to create field object based on value type
@@ -514,11 +561,11 @@ class BaseDynamicModel(BaseModel):
         """
         if isinstance(data, (dict, Mapping)):
             for key, value in data.items():
-                setattr(self, key, value)
+                if not key.startswith('__'):
+                    setattr(self, key, value)
 
 
 class DynamicModel(BaseDynamicModel):
-
     """
     DynamicModel allow to create model with no structure. Each instance has its own
     derivated class from DynamicModels.
@@ -526,6 +573,10 @@ class DynamicModel(BaseDynamicModel):
 
     _next_id = 0
 
+    def __init__(self, *args, **kwargs):
+        super(DynamicModel, self).__init__(*args, **kwargs)
+        self._structure = {}
+
     def __new__(cls, *args, **kwargs):
         new_class = type('DynamicModel_' + str(cls._next_id), (cls,), {'_dynamic_model': DynamicModel})
         cls._next_id = id(new_class)
@@ -537,6 +588,7 @@ class DynamicModel(BaseDynamicModel):
                 field_type = self._get_field_type(name, value)
                 if not field_type:
                     return
+                self._structure[field_type.name] = field_type
                 setattr(self.__class__, name, field_type)
 
         super(DynamicModel, self).__setattr__(name, value)
@@ -577,7 +629,6 @@ def recover_hashmap_model_from_data(model_class, original_data, modified_data, d
 
 
 class HashMapModel(InnerFieldTypeMixin, BaseModel):
-
     """
     Hash map model with dirty feature. It stores original data and saves
     modifications in other side.
@@ -660,7 +711,6 @@ class HashMapModel(InnerFieldTypeMixin, BaseModel):
 
 
 class FastDynamicModel(BaseDynamicModel):
-
     """
     FastDynamicModel allow to create model with no structure.
     """
@@ -689,6 +739,16 @@ class FastDynamicModel(BaseDynamicModel):
         else:
             return None
 
+    def get_current_structure(self):
+        """
+        Returns a dictionary with model field objects.
+        :return: dict
+        """
+
+        struct = self.__class__.get_structure()
+        struct.update(self._field_types)
+        return struct
+
     def __setattr__(self, name, value):
         if self._field_types is not None and not self.__hasattr__(name) \
                 and (not self.get_read_only() or not self.is_locked()):
